# -*- coding: utf-8 -*-

import unittest

import sys
sys.path.append('..')
from Rect import Rect

class TestRect(unittest.TestCase):

    def setUp(self):
        self.goodRect = Rect(1.5, 1.6, 3, 4)
        self.badRect = Rect(1.5, 1.6, 1.4, 1.6)

    def testWidth(self):
        self.assertAlmostEqual(self.goodRect.width(), 1.5)

    def testHeight(self):
        self.assertAlmostEqual(self.goodRect.height(), 2.4)

    def testArea(self):
        self.assertAlmostEqual(self.goodRect.area(), 3.6)

    def testOverlapTrue(self):
        rect1 = Rect(0, 0, 1, 1)
        rect2 = Rect(0.5, 0.5, 1.5, 1.5)
        self.assertTrue(rect1.isOverlap(rect2))

    def testOverlapFalse(self):
        rect1 = Rect(0, 0, 1, 1)
        rect2 = Rect(1, 0, 2, 1)
        self.assertFalse(rect1.isOverlap(rect2))

    def testInsideTrue(self):
        rect1 = Rect(0, 0, 1, 1)
        rect2 = Rect(-1, -1, 1.5, 1.5)
        self.assertTrue(rect1.isInside(rect2))

    def testInsideFalse(self):
        rect1 = Rect(0, 0, 1, 1)
        rect2 = Rect(0.5, 0.5, 1.5, 1.5)
        self.assertFalse(rect1.isInside(rect2))

    def testIntersection(self):
        rect3 = Rect(0, 0, 1, 1)
        rect4 = Rect(0.5, 0.5, 1.5, 1.5)
        rectInt2 = Rect.intersection(rect3, rect4)
        self.assertAlmostEqual(rectInt2.x0(), 0.5)
        self.assertAlmostEqual(rectInt2.y0(), 0.5)
        self.assertAlmostEqual(rectInt2.x1(), 1)
        self.assertAlmostEqual(rectInt2.y1(), 1)

    def testUnion(self):
        rect1 = Rect(0, 0, 1, 1)
        rect2 = Rect(0.5, 0.5, 1.5, 1.5)
        rectUnion = Rect.union(rect1, rect2)
        self.assertAlmostEqual(rectUnion.x0(), 0)
        self.assertAlmostEqual(rectUnion.y0(), 0)
        self.assertAlmostEqual(rectUnion.x1(), 1.5)
        self.assertAlmostEqual(rectUnion.y1(), 1.5)

    def testDistanceX(self):
        rect1 = Rect(0, 0, 1, 1)
        rect2 = Rect(1, 1, 2, 2)
        rect3 = Rect(2, 2, 3, 3)
        rect4 = Rect(0, 0, 1, 1)
        self.assertAlmostEqual(rect1.distanceX(rect2), 0)
        self.assertAlmostEqual(rect1.distanceX(rect3), 1)
        self.assertAlmostEqual(rect1.distanceX(rect4), -1)

    def testDistanceY(self):
        rect1 = Rect(0, 0, 1, 1)
        rect2 = Rect(1, 1, 2, 2)
        rect3 = Rect(2, 2, 3, 3)
        rect4 = Rect(0, 0, 1, 1)
        self.assertAlmostEqual(rect1.distanceY(rect2), 0)
        self.assertAlmostEqual(rect1.distanceY(rect3), 1)
        self.assertAlmostEqual(rect1.distanceY(rect4), -1)
        
    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()
